
package org.act.wsc.bpel4wsmodeling.view.graphic;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import javax.swing.JButton;
import javax.swing.plaf.basic.BasicButtonUI;
import org.act.wsc.bpel4wsmodeling.def.process.Activity;
import org.act.wsc.bpel4wsmodeling.global.Application;
import org.act.wsc.bpel4wsmodeling.projectdata.listener.IFigureUpdatedListener;
import org.act.wsc.bpel4wsmodeling.projectdata.process.Figure;

public class FigureViewer extends JButton implements IViewer,
		IFigureUpdatedListener {
	private FigureController myController = null;
	private boolean selected = false;
	private boolean debugged = false;
	
	
	
	
	protected Figure figure;
	
	public FigureViewer(Figure figure) {
		super();
		this.figure = figure;
		this.setUI(new BasicButtonUI());
		this.setBackground(Color.WHITE);
		this.setBorderPainted(false);
		this.setOpaque(true);
		this.setController();
		this.figure.addLayoutListeners(this);
	}
	
	
	public void setBound(){
		int w=this.figure.getWidth();
		int h=this.figure.getHeight();
		int x=this.figure.getX()-w/2;
		int y=this.figure.getY();
		this.setBounds(x,y,w,h);
	}
	
	
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Image image=null;
		if(figure.getOwner().isInDebug()&&figure.getServicesSet() instanceof Activity){
		    boolean isBreakpoint=((Activity)figure.getServicesSet()).isBreakPoint();
		    boolean isStop=figure.isStopped();
		    
		    if(isBreakpoint&&isStop){
		        if(this.figure.getStopDebugImage()!=null){
			        image=this.figure.getStopDebugImage().getImage();
			    }
		    }else if(isBreakpoint&&!isStop){
		        if(this.figure.getDebugImage()!=null){
			        image=this.figure.getDebugImage().getImage();
			    }
		    }else if(!isBreakpoint&&isStop){
		        if(this.figure.getStopImage()!=null){
			        image=this.figure.getStopImage().getImage();
			    }
		    }else{
		        image=this.figure.getImage().getImage();
		    }
		}
		else{
		    image=this.figure.getImage().getImage();
		}
		if(g!=null&&image!=null){		
			int w=this.figure.getImage().getIconWidth();
			int h=this.figure.getImage().getIconHeight();
			g.drawImage(image,0,0,w,h,null);
		}
	}
	
	public void figureLayoutUpdated() {
		this.setBound();
	}
	
	public void figurePropertyUpdated() {
			}
	
	public boolean isDebugged() {
		return debugged;
	}
	
	public void setDebugged(boolean debugged) {
		this.debugged = debugged;
	}
	
	public Figure getFigure() {
		return figure;
	}
	
	public void setFigure(Figure figure) {
		this.figure = figure;
	}
	
	public boolean isSelected() {
		return selected;
	}
	
	public void setSelected(boolean selected) {
		this.selected = selected;
	}
	
	public void setController() {
				this.myController = new FigureController(this.figure, this);
		this.addMouseListener(this.myController);
		this.addMouseMotionListener(this.myController);
	}
    
    public void figureDebugStateUpdated() {
        repaint();
    }
}
