
package org.act.wsc.bpel4wsmodeling.view.graphic;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.act.wsc.bpel4wsmodeling.debug.operate.DebugIdToFigure;
import org.act.wsc.bpel4wsmodeling.projectdata.listener.IFigureLifeCircleListener;
import org.act.wsc.bpel4wsmodeling.projectdata.listener.IFigureUpdatedListener;
import org.act.wsc.bpel4wsmodeling.projectdata.listener.ILayerUpdatedListener;
import org.act.wsc.bpel4wsmodeling.projectdata.process.Figure;
import org.act.wsc.bpel4wsmodeling.projectdata.process.GraphLayer;
import org.act.wsc.bpel4wsmodeling.projectdata.process.Line;
import org.act.wsc.bpel4wsmodeling.projectdata.process.StructFigure;

public class GraphLayerViewer extends LayerViewer implements
		IFigureLifeCircleListener, IFigureUpdatedListener,ILayerUpdatedListener {
	
	public void paint(Graphics g) {
				super.paint(g);
		if(this.lineToViewer.isEmpty()){
			return;
		}
				for (Iterator iter = this.lineToViewer.values().iterator(); iter
				.hasNext();) {
			((LineViewer) iter.next()).draw(g);
		}
				for (Iterator iter = this.borderToViewer.values().iterator(); iter.hasNext();) {
            ((BorderViewer) iter.next()).draw(g);
        }
	}
	private GraphLayerController myController = null;
	private Map figureToViewer=null;
	private Map lineToViewer=null;
	private Map borderToViewer=null;
	
	public GraphLayerViewer(GraphLayer layer) {
		super(layer);
				jbInit();
		this.lineToViewer=new HashMap();
		this.borderToViewer=new HashMap();
		this.figureToViewer=new HashMap();
		this.setController();
		layer.addListener(this);
		layer.addLayerUpdatedListener(this);
	}
	private void jbInit(){
		this.setLayout(null);
		this.setSize(((GraphLayer)layer).getWidth(), ((GraphLayer)layer).getHeight());
		this.setPreferredSize(new Dimension(((GraphLayer)layer).getWidth(), ((GraphLayer)layer).getHeight()));
		this.setBackground(Color.WHITE);
	}
	
	public GraphLayerViewer() {
		super();
			}
	
	protected GraphLayer getGraphLayer() {
		return (GraphLayer) layer;
	}
	
	protected void setGraphLayer(GraphLayer layer) {
		this.layer = layer;
	}
	public void addFigure(Class figureClass, Point point) {
		Figure figure;
	}
	public void removeFigure(FigureViewer figureViewer) {
	}
	public void addLine(LineViewer lineViewer) {
	}
	public void removeLine(LineViewer lineViewer) {
	}
	
	public void figureAdded(Figure figure) {
				this.addFigureViewer(figure);
		this.validate();
		this.repaint();
	}
	
	private void addFigureViewer(Figure figure) {
		FigureViewer figureViewer = new FigureViewer(figure);
		this.figureToViewer.put(figure, figureViewer);
        DebugIdToFigure.INSTANCE.addItem(figure);
		this.add(figureViewer);
		figureViewer.setBound();
		figure.addLayoutListeners(this);
		if (figure instanceof StructFigure) {
			for (Iterator iter = ((StructFigure) figure).getFigures()
					.iterator(); iter.hasNext();) {
				Figure childFigure = (Figure) iter.next();
				this.addFigureViewer(childFigure);
			}
			Figure endFigure = ((StructFigure) figure).getEnd();
			if (endFigure != null) {
				FigureViewer endFigureViewer = new FigureViewer(endFigure);
				this.figureToViewer.put(endFigure, endFigureViewer);
				this.add(endFigureViewer);
				endFigureViewer.setBound();
			}
			for (Iterator iter = ((StructFigure) figure).getLines().iterator(); iter
					.hasNext();) {
				Line line = (Line) iter.next();
				LineViewer lineViewer = new LineViewer(line);
				this.lineToViewer.put(line, lineViewer);
			}
			BorderViewer borderViewer=new BorderViewer((StructFigure) figure);
			if(((StructFigure) figure).getEnd()!=null){
			    this.borderToViewer.put((StructFigure) figure,borderViewer);
			}
			((StructFigure) figure).addListener(this);
		}
	}
	
	public void figureRemoved(Figure figure) {
				this.removeFigureViewer(figure);
		this.repaint();
		this.validate();
	}
	
	private void removeFigureViewer(Figure figure) {
		this.remove((FigureViewer) this.figureToViewer.get(figure));
		this.figureToViewer.remove(figure);
		DebugIdToFigure.INSTANCE.removeItem(figure);
		if (figure instanceof StructFigure) {
			for (Iterator iter = ((StructFigure) figure).getFigures()
					.iterator(); iter.hasNext();) {
				Figure childFigure = (Figure) iter.next();
				this.removeFigureViewer(childFigure);
			}
			Figure endFigure = ((StructFigure) figure).getEnd();
			if (endFigure != null) {
				this.remove((FigureViewer) this.figureToViewer.get(endFigure));
				this.figureToViewer.remove(endFigure);
			}
			for (Iterator iter = ((StructFigure) figure).getLines().iterator(); iter
					.hasNext();) {
				Line line = (Line) iter.next();
				this.lineToViewer.remove(line);
			}
			this.borderToViewer.remove(figure);
		}
	}
	
	public void lineAdded(Line line) {
				this.lineToViewer.put(line, new LineViewer(line));
		this.repaint();
	}
	
	public void lineRemoved(Line line) {
				this.lineToViewer.remove(line);
		this.repaint();
	}
	
	public void figureLayoutUpdated() {
		for (Iterator iter = this.figureToViewer.values().iterator(); iter.hasNext();) {
			FigureViewer element = (FigureViewer) iter.next();
			element.setBound();
		}
		this.repaint();
	}
	
	public void figurePropertyUpdated() {
			}
	
	public void setController() {
				this.myController = new GraphLayerController(this,
				(GraphLayer) this.layer);
		this.addMouseListener(this.myController);
		this.addMouseMotionListener(this.myController);
	}
    
    public void layerSizeChanged() {
        if(layer instanceof GraphLayer){
            this.setSize(((GraphLayer)layer).getWidth(),((GraphLayer)layer).getHeight());
            this.setPreferredSize(new Dimension(((GraphLayer)layer).getWidth(), ((GraphLayer)layer).getHeight()));
        }
    }
    
    public void figureDebugStateUpdated() {
        this.repaint();
    }
}
